import greenfoot.*;

/**
 * The world which has 7 platforms and a jumper in it.
 * All it can do is move all the platforms down a bit
 * (see the {@link movePlatforms} method)
 * <p>
 * If you use this class, please put my name in the credits (if you
 * have so) and put my name in the documentation:
 * 
 * @author Sven van Nigtevecht
 * @version 1.0.1
 */
public class JumpWorld extends World
{
    /** All the platforms where you can jump on */
    private Platform[] platforms = {
     new Platform(), new Platform(), new Platform(), new Platform(),
     new Platform(), new Platform(), new Platform()};
    
    /** Things for scrolling the background */
    private GreenfootImage backgroundScroll;
    private int backgroundScrollPosition;
    
    /**
     * Create a new JumpWorld.
     * This will add the platforms and the jumper.
     * It will also start the game.
     */
    public JumpWorld()
    {
        super(300, 450, 1);
        
        // In the original game, the background of the world was created in code,
        // which gives the result of random located stars.
        // I didn't do it here, because it isn't really special.
        // If you want the code, just ask me (on greenfoot.org)
        
        // add all the platforms (7):
        addObject(platforms[0], 313, 241);
        addObject(platforms[1], 130, 306);
        addObject(platforms[2], 339, 420);
        addObject(platforms[3], 119, 576);
        addObject(platforms[4], 479, 643);
        addObject(platforms[5], 249, 170);
        addObject(platforms[6], 452, 54);
        
        // create and add the jumper to the center of the world:
        addObject(new Jumper(), getWidth() /2, getHeight() /2);
        
        // create and add a FPS calculator:
        addObject(new FPS(), 85, 15);
        
        // start the program:
        Greenfoot.start();
    }
    
    /**
     * Moves all the platforms down.
     * This method is called by the jumper when it's at a
     * high location.
     * @param pixels The number of pixels that the platforms will move down
     */
    public void movePlatforms(int pixels)
    {
        // for each platform in the platforms array,
        for (Platform platform : platforms) {
            // add pixels to the y location of it
            // (which means it will move down, because
            // 0 is the top of the world of course :) )
            platform.setLocation(platform.getX(), platform.getY() +pixels);
        }
    }
    
    /**
     * let the background scroll upwards for the given pixels.
     * @param scroll How much the background will scroll (to scroll downwards enter
     * a negative number).
     * @author Busch2207 (from greenfoot.org)
     */
    public void scroll(int scroll)
    {
        if (scroll == 0) return;
        GreenfootImage back = getBackground();
        if (backgroundScroll == null) {
            backgroundScroll = new GreenfootImage(back.getWidth(),
                                                  back.getHeight() *2);
            backgroundScroll.drawImage(back, 0, 0);
            backgroundScroll.drawImage(back, 0, back.getHeight());
        }
        
        final int height = getHeight();
        backgroundScrollPosition += scroll;
        while (backgroundScrollPosition < 0) {
            backgroundScrollPosition += height;
        }
        backgroundScrollPosition %= height;
        back.drawImage(backgroundScroll, 0,
                       backgroundScrollPosition -height);
    }
}